import jieba
import jieba.posseg
from jieba.analyse.analyzer import ChineseAnalyzer, ChineseTokenizer
from os import path, getcwd
jieba.initialize()
d = path.dirname(__file__) if "__file__" in locals() else getcwd()
fontpath = path.join(d, r"googlefont", r"NotoSerifCJKtc-Light.otf")

chdictpath = path.join(d, r"chdict.txt")
jieba.load_userdict(chdictpath)
print("succesfully loaded userdict")

chstopwordspath = path.join(d, r"chstopwords.txt")
jieba.analyse.set_stop_words(chstopwordspath)
print("succesfully loaded stopwords")

def cutstopwords(text=[]):
    if isinstance(text, list) is False: return "call textcut in advance"
    with open(chstopwordspath, "r") as f:
        stopwords = f.read().splitlines()
        text = [_ for _ in text if _ not in stopwords]
    return text


# 精確模式
def textcut(io, excludeflag=["x"], removestopwords=True):
    if isinstance(io, list) is True: io=" ".join(io)
    if path.exists(io) is True:
        with open(io, "r") as f:
            f = ",".join(f)
            res = jieba.posseg.lcut(f)
    else:
        res = jieba.posseg.lcut(io)
    res = [_.word for _ in res if _.flag not in excludeflag]
    if removestopwords is True: res = cutstopwords(res)
    return res


def addword(io=[], freq=None, tag=None):
    def add(text, f=freq, t=tag):
        jieba.add_word(text, freq=f, tag=t)
        jieba.suggest_freq(text, tune=True)
        return
    if isinstance(io, str):
        add(io, f=freq, t=tag)
    elif isinstance(io, (list, set, tuple)):
        for _ in io :
            add(_ , f=freq, t=tag)
    return

if __name__ == "__main__":
    t = "氣象局指出，今天（15日）在西南風影響下，清晨至上午中南部沿海會有零星短暫陣雨，其他地區上半天雖然大致維持多雲到晴，不過由於環境水氣增多，天氣趨於不穩定，午後雷陣雨範圍將會擴大，除東半部地區及西半部山區外，大台北地區及西半部山區附近平地也有午後雷雨發生的機率，午後對流出現的時間會比較早，降雨有機會持續到晚上，並且局部地區也有短時強降雨發生的機率，請多留意午後天氣的變化。溫度方面，各地白天暖熱，高溫普遍約30到33度，南部近山區平地有機會出現36度以上高溫，中午前後請多補充水分及做好防曬措施，而各地夜晚及清晨低溫普遍在22至25度之間，相對舒適許多"
    # jieba.lcut(t)

    addword(["中南部沿海", "舒適許多"], tag="n")
    a=textcut(t)
    print(a)
    pass
